"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Promise = require("bluebird");
var chokidar = require("chokidar");
var tcGlob = require("./tsconfig-glob");
var U = require("./util");
var P = require("esdeps-parser");
// what is it that we are looking for?
var JsWatcher = /** @class */ (function () {
    function JsWatcher(options) {
        var _this = this;
        this.config = options.config;
        this.logger = options.logger.pushScope("JsWatcher");
        this._watcher = new chokidar.FSWatcher({
            persistent: true,
            depth: 0,
            ignorePermissionErrors: true
        });
        this._watcher
            .on('add', function (filePath) {
            _this._onAddFile(filePath);
        })
            .on('change', function (filePath) {
            _this._onChangeFile(filePath);
        })
            .on('unlink', function (filePath) {
            _this._onUnlinkFile(filePath);
        })
            .on('addDir', function (dirPath) {
            _this._onAddDir(dirPath);
        })
            .on('unlinkDir', function (filePath) {
            _this._onUnlinkDir(filePath);
        })
            .on('error', function (e) {
            _this._onError(e);
        });
        this._watcherSpec = new JsWatcherFileSpec(options);
    }
    JsWatcher.prototype.run = function () {
        var _this = this;
        // how do I know when I am done watch files?
        // that's something difficult to answer.
        return new Promise(function (resolve, reject) {
            if (_this.config.rootDir !== _this.config.outDir) {
                _this._watcher.add(_this._watcherSpec.includeDirs());
            }
            resolve();
        });
    };
    JsWatcher.prototype.getWatched = function () {
        var watched = this._watcher.getWatched();
        return watched;
    };
    JsWatcher.prototype._onAddFile = function (filePath) {
        this.logger.debug({
            scope: "_onAddFile",
            args: [filePath],
            isIgnored: this.config.isIgnoredPath(filePath),
            matches: {
                dotJs: this._isDotJs(filePath),
                dotDDotTs: this._isDotDts(filePath)
            },
            shouldWatch: this._shouldWatchFile(filePath)
        });
        if (this._shouldWatchFile(filePath)) {
            // we should copy to destination.
            copyFile(this.config, filePath, this.logger).then(function () { return null; });
        }
        else {
            this._watcher.unwatch(filePath);
        }
    };
    JsWatcher.prototype._onChangeFile = function (filePath) {
        this.logger.debug({
            scope: "_onChangeFile",
            args: [filePath],
            isIgnored: this.config.isIgnoredPath(filePath),
            matches: {
                dotJs: this._isDotJs(filePath),
                dotDDotTs: this._isDotDts(filePath)
            },
            shouldWatch: this._shouldWatchFile(filePath)
        });
        if (this._shouldWatchFile(filePath)) {
            // we should copy to destination.
            copyFile(this.config, filePath, this.logger).then(function () { return null; });
        }
        else {
            this._watcher.unwatch(filePath);
        }
    };
    JsWatcher.prototype._onUnlinkFile = function (filePath) {
        this.logger.debug({
            scope: '_onUnlinkFile',
            args: [filePath]
        });
        this._watcher.unwatch(filePath);
        var destPath = this.config.toOutPath(filePath);
        U.rmrf(destPath).then(function () { return null; });
    };
    JsWatcher.prototype._onUnlinkDir = function (filePath) {
        this.logger.debug({
            scope: '_onUnlinkDir',
            args: [filePath]
        });
        this._watcher.unwatch(filePath);
        var destPath = this.config.toOutPath(filePath);
        U.rmrf(destPath).then(function () { return null; });
    };
    JsWatcher.prototype._onError = function (e) {
        this.logger.error({
            scope: '_onError',
            error: e
        });
    };
    JsWatcher.prototype._onAddDir = function (dirPath) {
        this.logger.debug({
            scope: "_onAddDir",
            args: [dirPath],
            isIgnored: this.config.isIgnoredPath(dirPath)
        });
        if (!this.config.isIgnoredPath(dirPath)) {
            this._watcher.add(dirPath);
        }
    };
    JsWatcher.prototype._shouldWatchFile = function (filePath) {
        return (this._isDotDts(filePath) || this._isDotJs(filePath)) && !this.config.isIgnoredPath(filePath);
    };
    JsWatcher.prototype._isDotJs = function (filePath) {
        return filePath.toLowerCase().endsWith('.js');
    };
    JsWatcher.prototype._isDotDts = function (filePath) {
        return filePath.toLowerCase().endsWith('.d.ts');
    };
    return JsWatcher;
}());
exports.JsWatcher = JsWatcher;
function copyFile(config, filePath, logger) {
    var outPath = config.toOutPath(filePath);
    // what do I want to do? I want to read in the file and
    // then do some transformation.
    return U.readFile(filePath)
        .then(function (data) { return P.parse({
        filePath: filePath, data: data
    }); })
        .then(function (mod) {
        mod.depends.forEach(function (dep) {
            // what do I want to do?
            // if depend == source-map-support - delete this depend.
            // if depend has outDir, replace it.
            // this.logger.debug({
            //     scope: `_copyFile:DEPEND`,
            //     dep,
            //     isFilter: dep.spec.match('source-map-support'),
            //     isRelative: dep.isRelative,
            //     normalizedSpec : dep.isRelative ? this.config.moveModuleSpec(mod.filePath, dep.spec) : dep.spec
            // })
            if (dep.spec.match('source-map-support')) {
                mod.removeDep(dep);
            }
            else if (dep.spec.match('ts-node')) {
                mod.removeDep(dep);
            }
            else if (dep.isRelative) {
                mod.moveDep(dep, config.moveModuleSpec(mod.filePath, dep.spec));
            } // else nothing.
        });
        return mod.toString();
    })
        .then(function (data) {
        return U.writeFile(outPath, data);
    })
        .catch(function (e) {
        logger.error({
            scope: "copyFiles",
            error: e
        });
    });
}
exports.copyFile = copyFile;
var JsWatcherFileSpec = /** @class */ (function () {
    function JsWatcherFileSpec(options) {
        this.config = options.config;
    }
    JsWatcherFileSpec.prototype.includeDirs = function () {
        var _this = this;
        return (this.config.include || []).concat(this.config.files || []).map(function (spec) {
            return new tcGlob.TsConfigGlob({
                spec: spec,
                basePath: _this.config.rootPath,
                allowTypes: ['.js', '.d.ts']
            });
        })
            .filter(function (glob) {
            return glob.isDirectorySpec() ||
                glob.isWildCardSpec() ||
                (glob.isFileSpec() && (glob.hasExtension('.js') || glob.hasExtension('.d.ts')));
        })
            .map(function (glob) {
            return glob.toJsWatcherDirPath(true);
        });
    };
    JsWatcherFileSpec.prototype.isIgnoredPath = function (filePath) {
        var _this = this;
        var globs = (this.config.excluded || []).map(function (spec) {
            return new tcGlob.TsConfigGlob({
                spec: spec,
                basePath: _this.config.rootPath
            });
        });
        var glob = globs.find(function (glob) {
            return glob.match(filePath);
        });
        return !!glob;
    };
    return JsWatcherFileSpec;
}());
exports.JsWatcherFileSpec = JsWatcherFileSpec;
//# sourceMappingURL=js-watcher.js.map