"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var vpath = require("./vpath");
var TsConfigGlob = /** @class */ (function () {
    function TsConfigGlob(options) {
        this.spec = new vpath.PathObject(options.spec);
        this.basePath = new vpath.PathObject(options.basePath || process.cwd());
        this.allowTypes = options.allowTypes ? options.allowTypes : options.allowJs ? ['.ts', '.tsx', '.js', '.jsx'] : ['.ts', '.tsx'];
    }
    TsConfigGlob.prototype.toFullPath = function () {
        return this.basePath.join(this.spec.toVirtualPath());
    };
    TsConfigGlob.prototype.toRegExp = function (useFullPath) {
        var _this = this;
        if (useFullPath === void 0) { useFullPath = false; }
        // we are matching on / as the separator.
        if (useFullPath) {
            var segs = this.basePath.segments().concat(this.spec.segments());
            return new RegExp("^" + segs.map(function (seg) { return _this._segToRegEx(seg); }).join('\\/'));
        }
        else {
            var segs = this.spec.segments();
            return new RegExp("\\/" + segs.map(function (seg) { return _this._segToRegEx(seg); }).join('\\/'));
        }
    };
    TsConfigGlob.prototype.match = function (filePath) {
        filePath = new vpath.PathObject(filePath).toVirtualPath();
        var matched = filePath.match(this.toRegExp());
        return !!matched;
    };
    TsConfigGlob.prototype.isRecursiveSpec = function () {
        var segments = this.spec.segments();
        var seg = segments.find(function (seg) { return seg === '**'; });
        return !!seg;
    };
    TsConfigGlob.prototype.isFileSpec = function () {
        var extname = this.spec.extname();
        return extname !== '';
    };
    TsConfigGlob.prototype.hasExtension = function (extname) {
        return this.spec.lastSeg().endsWith(extname);
    };
    TsConfigGlob.prototype.hasWildCard = function () {
        var segments = this.spec.segments();
        var seg = segments.find(function (seg) {
            return seg !== '**' && (seg.indexOf('*') !== -1 || seg.indexOf('?') !== -1);
        });
        return !!seg;
    };
    TsConfigGlob.prototype.isDirectorySpec = function () {
        return !this.isWildCardSpec() && this.spec.extname() === '';
    };
    TsConfigGlob.prototype.isWildCardSpec = function (seg) {
        if (seg === void 0) { seg = this.spec.lastSeg(); }
        return seg === '*' || seg === '**';
    };
    TsConfigGlob.prototype.toIncludeGlob = function (useFullPath) {
        // if ends in directory name.
        return this._useFullPath(this._toIncludeGlob(), useFullPath || false);
    };
    TsConfigGlob.prototype.toExcludeGlob = function (useFullPath) {
        return this._useFullPath(this._toExcludeGlob(), useFullPath || false);
    };
    TsConfigGlob.prototype.toJsWatcherDirPath = function (useFullPath) {
        if (useFullPath === void 0) { useFullPath = true; }
        // what are we looking for? the following.
        // a single dir path
        if (this.isDirectorySpec()) {
            return this._useFullPath(this.spec.toVirtualPath(), useFullPath, false);
        }
        else if (this.isWildCardSpec()) {
            // wildcard spec ends in *, and or could be a directory.
            // strip out the parts that are mean
            return this._useFullPath(this._stripWildCard(), useFullPath, false);
        }
        else { // if this is a file - what do we do? return itself.
            return this._useFullPath(this.spec.toVirtualPath(), useFullPath, false);
        }
    };
    TsConfigGlob.prototype._toExcludeGlob = function () {
        if (this.isDirectorySpec()) {
            return this.spec.toVirtualPath() + "/**/*";
        }
        else { // this should have been basically a 
            return this.spec.toVirtualPath();
        }
    };
    TsConfigGlob.prototype._stripWildCard = function () {
        if (this.hasWildCard()) {
            var segments = this.spec.segments();
            segments.pop();
            return new vpath.PathObject(segments.join(vpath.BASE_DELIM)).toVirtualPath();
        }
        else {
            return this.spec.toVirtualPath();
        }
    };
    TsConfigGlob.prototype._toIncludeGlob = function () {
        // we want to exclude the directory itself!!!
        if (this.isDirectorySpec()) {
            return this.spec + "/**/*." + this.globAllowTypeString();
        }
        else if (this.isWildCardSpec()) {
            // what to do otherwise?
            // we can make assumption that 
            return this.spec + '.' + this.globAllowTypeString();
        }
        else { // this should have been basically a 
            return this.spec.toVirtualPath();
        }
    };
    TsConfigGlob.prototype.globAllowTypeString = function () {
        return '{' + this.allowTypes.map(function (ext) { return ext.substring(1); }).join(',') + '}';
    };
    TsConfigGlob.prototype._useFullPath = function (spec, useFullPath, useVPath) {
        if (useVPath === void 0) { useVPath = true; }
        if (useFullPath) {
            var joinedPath = this.basePath.join(spec);
            if (useVPath) {
                return joinedPath.toVirtualPath();
            }
            else {
                return joinedPath.toOsPath();
            }
        }
        else {
            return spec;
        }
    };
    TsConfigGlob.prototype._segToRegEx = function (seg) {
        // https://www.typescriptlang.org/docs/handbook/tsconfig-json.html
        // * matches zero or more characters (excluding directory separators)
        // ? matches any one character (excluding directory separators)
        // **/ recursively matches any subdirectory
        if (seg === '**') {
            return "(.*)?";
        }
        else {
            return seg.replace(/\?/g, '[^\/]?')
                .replace(/\*/g, '([^\/]*)?')
                .replace(/\./g, '\\.');
        }
    };
    return TsConfigGlob;
}());
exports.TsConfigGlob = TsConfigGlob;
//# sourceMappingURL=tsconfig-glob.js.map